/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2020                               **
**                                                                        **
**                             UDP example                                **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2020.11.26                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include <LPC23xx.H>
#include "system.h"
#include "types.h"
#include "systime.h"
#include "irqvectors.h"
#include "serialconst.h"
#include "uart0.h"
#include "strings.h"
#include "emacdriver.h"
#include "ipservice.h"
#include "udpservice.h"
#include "trace.h"

#define IRQCallCntLimit                 500
#define SerialBufferSize                128

typedef struct {
                 EMACInstanceRecT *          EMACLayer ;
                 IPLayerInstanceRecT *       IPLayer ;
                 PtrUDPLayerInstanceRecT     UDPLayer ;
                 PtrUDPSocketDescriptionRecT UDPSocket ;
                 PtrTimeQueueRecT            TimerAction ;
                 UCHAR                       SerialBuffer [ SerialBufferSize ] ;
                 USHORT                      SerialBufferIndex ;
                 ULONG                       LastClientIPAddress ;
                 USHORT                      LastClientPortNo ;
               } UDPExampleInstanceRecT ;
typedef UDPExampleInstanceRecT * PtrUDPExampleInstanceRecT ;

static UDPExampleInstanceRecT UDPDemoInstance ; ;
static ULONG IRQCallCounter ;

#define NetBufferSize                256

#define MachineIPAddress        0xC0A80037
//                              ^^^ 192.168.0.55
#define MachineIPSubnet         0xFFFFFF00
//                              ^^^ 255.255.255.0
#define MachineGWIPAddress      0xC0A800FE
//                              ^^^ 192.168.0.254
#define UDPPortNumber           20001

UCHAR MachineMACAddr [ 6 ]     = { 0x08 , 0x14 , 0x16 , 0x22 , 0x28 , 0x2D } ;

static UCHAR NoUDPData [ ]     = "Pusty pakiet UDP" ;
static UCHAR Error1 [ ]        = "\r\nProblem z tworzeniem gniazdka\r\n" ;
static UCHAR Error2 [ ]        = "\r\nProblem z nadaniem numeru gniazdku\r\n" ;
static UCHAR NoDestMsg [ ]     = "\r\nNieznane jest miejsce docelowe\r\n" ;
static UCHAR SendError [ ]     = "\r\nCos poszlo nie tak...\r\n" ;

static __irq void T1_IRQHandler ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter ++ ;
  if ( IRQCallCounter >= IRQCallCntLimit )
  {
    IRQCallCounter = 0 ;
  } /* if */ ;
  T1IR = 1 ;
  VICVectAddr = 0 ;
} /* T1_IRQHandler */


static void HardwareInit ( void ) 
{
  /*-------------------------------------------------------------------------*/
  PINSEL10 = 0 ;
  T1MR0 = ( PCLK / 1000 ) - 1 ;
  T1MCR = 3 ;
  T1TCR = 1 ;
  VICVectAddr5 = ( unsigned long ) T1_IRQHandler ;
  VICVectCntl5 = 15 ;
  VICIntEnable = ( 1 << TIMER1_Interrupt ) ;
  UART0InitEnvir ( ) ;
} /* HardwareInit */


static void StartNet ( PtrUDPExampleInstanceRecT Instance )
{
  EtherSpeedMode EthMode ;
  /*-------------------------------------------------------------------------*/
  EthMode = Mode_AutoNeg ;
//  EthMode = Mode_10MBIT ;
//  EthMode = Mode_100MBIT ;
  Instance -> EMACLayer = InitPHYLayerInstance ( MachineMACAddr , EthMode ) ;
  Instance -> IPLayer = InitIPLayerInstance ( Instance -> EMACLayer ,
                                              ( ULONG ) MachineIPAddress ,
                                              ( ULONG ) MachineIPSubnet ) ;
  Instance -> IPLayer -> MyDefaultGWIPAddress = ( ULONG ) MachineGWIPAddress ;
  Instance -> UDPLayer = StandardAddUDPInstance ( Instance -> IPLayer ) ;
} /* StartNet */


UCHAR Hello1 [ ]      = "\r\nHello: serwer UDP\r\n" ;
UCHAR Hello2 [ ]      = "\r\nMoj adres IP=" ;
UCHAR Hello3 [ ]      = "\r\nMaska podsieci=" ;
UCHAR Hello4 [ ]      = "\r\nAdres bramy domyslnej=" ;
UCHAR Hello6 [ ]      = "\r\nAdres MAC=" ;
UCHAR Hello7 [ ]      = "\r\n************************\r\n" ;

static void HelloMessage ( void )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Hello1 ) ;
  UART0SendString ( Hello2 ) ;
  TraceIP ( MachineIPAddress ) ;
  UART0SendString ( Hello3 ) ;
  TraceIP ( MachineIPSubnet ) ;
  UART0SendString ( Hello4 ) ;
  TraceIP ( MachineGWIPAddress ) ;
  UART0SendString ( Hello6 ) ;
  TraceMAC ( MachineMACAddr ) ;
  UART0SendString ( Hello7 ) ;
} /* HelloMessage */


static void SoftwareInit ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter = 0 ;
  UDPDemoInstance . EMACLayer = NIL ;
  UDPDemoInstance . IPLayer = NIL ;
  UDPDemoInstance . UDPLayer = NIL ;
  UDPDemoInstance . UDPSocket = NIL ;
  UDPDemoInstance . TimerAction = NIL ;
  UDPDemoInstance . SerialBufferIndex = 0 ;
  UDPDemoInstance . LastClientIPAddress = 0 ;
  UDPDemoInstance . LastClientPortNo = 0 ;
} /* SoftwareInit */


static void UDPStreamService ( PtrUDPSocketDescriptionRecT UDPSocket ,
                               ULONG                       SenderIPAddress ,
                               USHORT                      SenderPortNo ,
                               ULONG                       DestIPAddress ,
                               USHORT                      DestPortNo ,
                               void *                      UDPDataBuffer ,
                               USHORT                      UDPDataSize )
{
  UCHAR * ChPtr ;
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  UDPDemoInstance . LastClientIPAddress = SenderIPAddress ;
  UDPDemoInstance . LastClientPortNo = SenderPortNo ;
  if ( UDPDataSize == 0 )
  {
    UART0SendString ( NoUDPData ) ;
  } /* if ... */
  else
  {
    ChPtr = ( UCHAR * ) UDPDataBuffer ;
    for ( Loop = 0 ; Loop < UDPDataSize ; Loop ++ )
    {
      if ( ( ( * ChPtr ) >= 0x20 ) && ( ( * ChPtr ) < 0x80 ) )
        UART0Send ( * ChPtr ) ;
      else
        UART0Send ( '.' ) ;
      ChPtr ++ ;
    } /* for */ ;
  } /* if ... else */ ;
  UART0Send ( '\r' ) ;
  UART0Send ( '\n' ) ;
} /* UDPStreamService */


static void SendBuffer ( void * CallParam )
{
  USHORT SendReply ;
  /*-------------------------------------------------------------------------*/
  if ( ! UDPDemoInstance . LastClientIPAddress )
  {
    UART0SendString ( NoDestMsg ) ;
  } /* if ... */ 
  else
  {
    SendReply = UDPSendData ( UDPDemoInstance . UDPSocket ,
                              UDPDemoInstance . LastClientIPAddress ,
                              UDPDemoInstance . LastClientPortNo ,
                              ( void * ) UDPDemoInstance . SerialBuffer ,
                              UDPDemoInstance . SerialBufferIndex ) ;
    if ( SendReply != UDP_SendOK )
      UART0SendString ( SendError ) ;
  } /* if ... else */ ;
  UDPDemoInstance . SerialBufferIndex = 0 ;
} /* SendBuffer */


static void SerialPool ( void )
{
  UCHAR Ch ;
  /*-------------------------------------------------------------------------*/
  if ( UART0DataPresent ( ) )
  {
    Ch = UART0GetData ( ) ;
    UDPDemoInstance . SerialBuffer [ UDPDemoInstance . SerialBufferIndex ] = Ch ;
    UDPDemoInstance . SerialBufferIndex ++ ;
    DetachTimer ( UDPDemoInstance . TimerAction ) ;
    if ( UDPDemoInstance . SerialBufferIndex >= SerialBufferSize )
    {
      SendBuffer ( NIL ) ;
    } /* if ... */ 
    else
    {
      UDPDemoInstance . TimerAction = AttachTimer ( 100 , SendBuffer , NIL ) ;
    } /* if ... else */ ;
  } /* if */ ;
} /* SerialPool */


static void Halt ( UCHAR * Message )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Message ) ;
  for ( ; ; )
  {
    NOP ( ) ;
  } /* loop */ ;
} /* Halt */


int main ( void )
{
  USHORT BindStatus ;
  /*-------------------------------------------------------------------------*/
  SysInit ( ) ;
  InitSysTime ( ) ;
  HardwareInit ( )  ;
  SoftwareInit ( ) ;
  UART0HardwInit( FOSC , N81Mode , 9600 ) ;
  HelloMessage ( ) ;
  StartNet ( & UDPDemoInstance ) ;
  UDPDemoInstance . UDPSocket =  CreateUDPSocket ( UDPDemoInstance . UDPLayer ) ;
  if ( ! UDPDemoInstance . UDPSocket )
    Halt ( Error1 ) ;
  BindStatus = BindUDPSocket ( UDPDemoInstance . UDPSocket , UDPPortNumber ) ;
  if ( BindStatus != UDP_BindPortOK )
  {
    Halt ( Error2 ) ;
  } /* if */ ;
  SetUDPListen ( UDPDemoInstance . UDPSocket , UDPStreamService ) ;
  for ( ; ; )
  {
    SysTimerPool ( ) ;
    UDPDemoInstance . IPLayer -> IPLayerPoolService ( UDPDemoInstance . IPLayer ) ;
    SerialPool ( ) ;
  } /* loop */ ;
} /* main */

